
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.17.6.post2',
    'branch_nick': '(HEAD',
    'revision_id': '1515c9dba4a8fd7acf6b6a2c63c5d40a936ca924',
    'revno': 6899
}
